#!/usr/bin/env python

import os
import sys
import urllib
import timeit
import signal
import subprocess
import ConfigParser

process = None

def signal_handler(signum, frame):
    global process
    sys.stderr.write("Handle signal %d, kill datadome_proxyserver and quit\n" % signum)
    process.kill()
    sys.exit()

def autodetection(config, debug):
    times = 10
    url = ''
    stats = ''

    if config.has_option('AutoDetection', 'times'):
        times = config.getint('AutoDetection', 'times')

    if config.has_option('AutoDetection', 'url'):
        url = config.get('AutoDetection', 'url')

    if config.has_option('AutoDetection', 'stats'):
        stats = config.get('AutoDetection', 'stats')

    if debug:
        print("Run auto detection on URL: %s, stats: %s, times: %d" % (url, stats, times))

    if times <= 0 or len(url) == 0:
        return False, ''

    try:
        results = {}
        response = urllib.urlopen(url)
        servers = response.read().splitlines()
        for server in servers:
            if debug:
                print("Testing server %s..." % server)
            try:
                results[server] = (timeit.timeit('urllib.urlopen("http://%s/status")' % server, number=times, setup='import urllib') / times * 1000)
                if debug:
                    print("Ok, %f ms average response time" % results[server])
            except:
                pass

        if len(results) == 0:
            return False, ''

        best = min(results, key=results.get)
        if debug:
            print("The beset server is %s" % best)
        if len(stats) > 0:
            try:
                urllib.urlopen(stats, data=urllib.urlencode({'stats': results, 'from': 'proxyserver'}))
            except:
                pass
        return True, best
    except:
        return False, ''


def main():
    signal.signal(signal.SIGHUP, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    debug = False

    cmd = []

    if (len(sys.argv) == 2 or len(sys.argv) == 3) and sys.argv[1] != '--pidfile':
        cmd += [sys.argv[1]]
    else:
        if len(sys.argv) == 5 or len(sys.argv) == 4:
            cmd += [sys.argv[3]]
        else:
            sys.stderr.write("Usage: %s [--pidfile /path/to/file.pid] </path/to/datadome_proxyserver> [config]\n" % sys.argv[0])
            sys.exit(1)

    pid = None
    if sys.argv[1] == '--pidfile':
        pid = sys.argv[2]
        f = open(pid, 'w')
        f.write("%d" % os.getpid())
        f.close()

    if (len(sys.argv) == 3 and pid is None) or len(sys.argv) == 5:
        config = ConfigParser.ConfigParser()
        if len(sys.argv) == 3:
            config.read(sys.argv[2])
        else:
            config.read(sys.argv[4])
        autodetected = False

        debug = (config.has_option('WatchDog', 'debug') and config.getboolean('WatchDog', 'debug'))

        if config.has_option('ProxyServer', 'local_address'):
            cmd += ['--local_address', config.get('ProxyServer', 'local_address')]

        if config.has_option('ProxyServer', 'local_port'):
            cmd += ['--local_port', config.get('ProxyServer', 'local_port')]

        if config.has_option('ProxyServer', 'auto_detection') and config.getboolean('ProxyServer', 'auto_detection'):
            (autodetected, remote_address) = autodetection(config, debug)
            if autodetected:
                cmd += ['--remote_address', remote_address]

        if config.has_option('ProxyServer', 'remote_address') and autodetected == False:
            cmd += ['--remote_address', config.get('ProxyServer', 'remote_address')]

        if config.has_option('ProxyServer', 'remote_port'):
            cmd += ['--remote_port', config.get('ProxyServer', 'remote_port')]

        if config.has_option('ProxyServer', 'ssl') and config.getboolean('ProxyServer', 'ssl'):
            cmd += ['--ssl']

        if config.has_option('ProxyServer', 'async') and config.getboolean('ProxyServer', 'async'):
            cmd += ['--async']

        if config.has_option('ProxyServer', 'forbidden') and config.getboolean('ProxyServer', 'forbidden'):
            cmd += ['--forbidden']

        if config.has_option('ProxyServer', 'remote_timeout'):
            cmd += ['--remote_timeout', config.get('ProxyServer', 'remote_timeout')]

        if config.has_option('ProxyServer', 'remote_reconnect_timeout'):
            cmd += ['--remote_reconnect_timeout', config.get('ProxyServer', 'remote_reconnect_timeout')]

        if config.has_option('ProxyServer', 'max_remote_connections'):
            cmd += ['--max_remote_connections', config.get('ProxyServer', 'max_remote_connections')]

        if config.has_option('ProxyServer', 'max_messages_queue_length'):
            cmd += ['--max_messages_queue_length', config.get('ProxyServer', 'max_messages_queue_length')]

        if config.has_option('ProxyServer', 'threads'):
            cmd += ['--threads', config.get('ProxyServer', 'threads')]

        if config.has_option('ProxyServer', 'client_id'):
            cmd += ['--client_id', config.get('ProxyServer', 'client_id')]

        if config.has_option('ProxyServer', 'proxy_address'):
            cmd += ['--proxy_address', config.get('ProxyServer', 'proxy_address')]

        if config.has_option('ProxyServer', 'proxy_port'):
            cmd += ['--proxy_port', config.get('ProxyServer', 'proxy_port')]

        if config.has_option('ProxyServer', 'debug') and config.getboolean('ProxyServer', 'debug'):
            cmd += ['--debug']

    global process
    while True:
        if debug:
            print("Run proxyserver by command")
            print(cmd)
        process = subprocess.Popen(cmd)
        process.wait()
        sys.stderr.write("Ops, datadome_proxyserver was killed or crashed, just restart it\n")

if __name__ == "__main__":
    main()
